@@ -9,10 +9,11 @@ def command(name, aliases=None, usage=None, description=None, roles=None, ignore
             setattr(func, 'aliases', aliases)
         if usage:
             setattr(func, 'usage', usage)
-        if roles:
-            setattr(func, 'roles', roles)
-        else:
-            setattr(func,'roles',[])
+        if roles is []:
+            setattr(func, 'roles', [])
+        elif roles is not None:
+            setattr(func,'roles',roles)
+
 
         return func
 
